package v1

import (
	pbpod "code.gitlink.org.cn/JCCE/PCM.git/adaptor/pcm_pod/gen/idl"
	"code.gitlink.org.cn/JCCE/PCM.git/adaptor/pcm_pod/server"
	"code.gitlink.org.cn/JCCE/PCM.git/adaptor/pcm_pod/server/kubernetes/api/core/v1"
	"code.gitlink.org.cn/JCCE/PCM.git/common/tenanter"
	pbtenant "code.gitlink.org.cn/JCCE/PCM.git/tenant/gen/idl"
	"context"
	"encoding/json"
	"errors"
	"flag"
	"fmt"
	"github.com/golang/glog"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"strconv"
	"time"

	"k8s.io/apimachinery/pkg/types"
	"k8s.io/apimachinery/pkg/watch"
	corev1 "k8s.io/client-go/applyconfigurations/core/v1"
	"k8s.io/client-go/kubernetes/scheme"
	"k8s.io/client-go/rest"
)

// PodsGetter has a method to return a PodInterface.
// A group's client should implement this interface.
type PodsGetter interface {
	Pods(namespace string) PodInterface
}

// PodInterface has methods to work with Pod resources.
type PodInterface interface {
	Create(ctx context.Context, pod *v1.Pod, opts metav1.CreateOptions) (*v1.Pod, error)
	Update(ctx context.Context, pod *v1.Pod, opts metav1.UpdateOptions) (*v1.Pod, error)
	UpdateStatus(ctx context.Context, pod *v1.Pod, opts metav1.UpdateOptions) (*v1.Pod, error)
	Delete(ctx context.Context, namespace string, providerId int32, regionName string, accountName string, pcmId string, opts metav1.DeleteOptions) error
	DeleteCollection(ctx context.Context, opts metav1.DeleteOptions, listOpts metav1.ListOptions) error
	Get(ctx context.Context, name string, opts metav1.GetOptions) (*v1.Pod, error)
	List(ctx context.Context, pod *v1.Pod, opts metav1.ListOptions) (*v1.PodList, error)
	Watch(ctx context.Context, opts metav1.ListOptions) (watch.Interface, error)
	Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts metav1.PatchOptions, subresources ...string) (result *v1.Pod, err error)
	Apply(ctx context.Context, pod *corev1.PodApplyConfiguration, opts metav1.ApplyOptions) (result *v1.Pod, err error)
	ApplyStatus(ctx context.Context, pod *corev1.PodApplyConfiguration, opts metav1.ApplyOptions) (result *v1.Pod, err error)
	GetEphemeralContainers(ctx context.Context, podName string, options metav1.GetOptions) (*v1.EphemeralContainers, error)
	UpdateEphemeralContainers(ctx context.Context, podName string, ephemeralContainers *v1.EphemeralContainers, opts metav1.UpdateOptions) (*v1.EphemeralContainers, error)

	PodExpansion
}

// pods implements PodInterface
type pods struct {
	client rest.Interface
	ns     string
}

// newPods returns a Pods
func newPods(c *CoreV1Client, namespace string) *pods {
	return &pods{
		client: c.RESTClient(),
		ns:     namespace,
	}
}

// Get takes name of the pcm_pod, and returns the corresponding pcm_pod object, and an error if there is any.
func (c *pods) Get(ctx context.Context, name string, options metav1.GetOptions) (result *v1.Pod, err error) {
	result = &v1.Pod{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("pods").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do(ctx).
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of Pods that match those selectors.
func (c *pods) List(ctx context.Context, pod *v1.Pod, opts metav1.ListOptions) (result *v1.PodList, err error) {

	providerId := pod.Spec.ProviderId
	regionId := pod.Spec.RegionId
	accountName := pod.Spec.AccountName
	provider := pbtenant.CloudProvider(providerId)

	requestPCM := &pbpod.ListPodReq{
		RequestSource: "huawei",
		Provider:      pbpod.CloudProvider(provider),
		Namespace:     "pcm",
	}

	resp, err := server.ListPod(nil, requestPCM)
	//trans PCM response pcm_pod set to Ali ContainerGroup set
	var pods = make([]v1.Pod, len(resp.Pods))
	for k := range resp.Pods {
		podId := resp.Pods[k].PodId
		containerName := resp.Pods[k].ContainerName
		containerImage := resp.Pods[k].ContainerImage
		podName := resp.Pods[k].PodName
		container := v1.Container{
			Name:  containerName,
			Image: containerImage,
		}
		var containers []v1.Container
		containers = append(containers, container)

		podSpec := v1.PodSpec{
			ProviderId:  providerId,
			RegionId:    regionId,
			AccountName: accountName,
			Containers:  containers,
		}
		pod := v1.Pod{
			TypeMeta:   metav1.TypeMeta{APIVersion: "core/V1", Kind: "Pod"},
			ObjectMeta: metav1.ObjectMeta{Name: podName, Namespace: "pcm", UID: types.UID(podId)},
			Spec:       podSpec,
			Status:     v1.PodStatus{},
		}
		pods[k] = pod
	}

	result = &v1.PodList{
		TypeMeta: metav1.TypeMeta{},
		ListMeta: metav1.ListMeta{},
		Items:    pods,
	}
	return
}

// Watch returns a watch.Interface that watches the requested pods.
func (c *pods) Watch(ctx context.Context, opts metav1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Namespace(c.ns).
		Resource("pods").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch(ctx)
}

// Create takes the representation of a pcm_pod and creates it.  Returns the server's representation of the pcm_pod, and an error, if there is any.
func (c *pods) Create(ctx context.Context, pod *v1.Pod, opts metav1.CreateOptions) (result *v1.Pod, err error) {
	result = &v1.Pod{}

	provider := pbtenant.CloudProvider(pod.Spec.ProviderId)
	var configFile string
	flag.StringVar(&configFile, "conf", "configs/tenanter.yaml", "tenanter.yaml")
	flag.Parse()
	defer glog.Flush()

	if err := tenanter.LoadCloudConfigsFromFile(configFile); err != nil {
		if !errors.Is(err, tenanter.ErrLoadTenanterFileEmpty) {
			glog.Fatalf("tenanter.LoadCloudConfigsFromFile error %+v", err)
		}
		glog.Warningf("tenanter.LoadCloudConfigsFromFile empty file path %s", configFile)
	}

	glog.Infof("load tenant from file finished")
	tenanters, err := tenanter.GetTenanters(provider)
	var regionId int32
	var cpuPod string
	var memoryPod string

	//华为(K8S)与其他厂商的CPU单位和格式有区别,region根据字符串取各厂商的对应id
	switch pod.Spec.ProviderId {
	case 0:
		regionId, _ = tenanter.GetAliRegionId(pod.Spec.RegionId)
		cpuPod = strconv.FormatInt(pod.Spec.Containers[0].Resources.Limits.Cpu().MilliValue()/1000, 10)
		memoryPod = strconv.FormatInt(pod.Spec.Containers[0].Resources.Limits.Memory().MilliValue()/1024/1024/1024/1000, 10)
	case 1:
		regionId, _ = tenanter.GetTencentRegionId(pod.Spec.RegionId)
		cpuPod = strconv.FormatInt(pod.Spec.Containers[0].Resources.Limits.Cpu().MilliValue()/1000, 10)
		memoryPod = strconv.FormatInt(pod.Spec.Containers[0].Resources.Limits.Memory().MilliValue()/1024/1024/1024/1000, 10)
	case 2:
		regionId, _ = tenanter.GetHuaweiRegionId(pod.Spec.RegionId)
		cpuPod = pod.Spec.Containers[0].Resources.Limits.Cpu().String()
		memoryPod = pod.Spec.Containers[0].Resources.Limits.Memory().String()
	case 3:
		regionId, _ = tenanter.GetK8SRegionId(pod.Spec.RegionId)
		cpuPod = strconv.FormatInt(pod.Spec.Containers[0].Resources.Limits.Cpu().MilliValue()/1000, 10)
		memoryPod = strconv.FormatInt(pod.Spec.Containers[0].Resources.Limits.Memory().MilliValue()/1024/1024/1024/1000, 10)
	}

	requestPCM := &pbpod.CreatePodReq{
		RequestSource:  "huawei",
		Provider:       pbpod.CloudProvider(provider),
		AccountName:    tenanters[0].AccountName(),
		PodName:        pod.Name,
		RegionId:       regionId,
		ContainerImage: pod.Spec.Containers[0].Image,
		ContainerName:  pod.Spec.Containers[0].Name,
		CpuPod:         cpuPod,
		MemoryPod:      memoryPod,
		//这里腾讯需要独立提供三个参数，目前固定使用以下三个
		SecurityGroupId: "sg-6qlun7hd",
		SubnetId:        "subnet-mnwfg2fk",
		VpcId:           "vpc-rkwt40g5",
		Namespace:       pod.ObjectMeta.Namespace,
	}

	server.CreatePod(nil, requestPCM)

	result = &v1.Pod{
		TypeMeta:   metav1.TypeMeta{},
		ObjectMeta: metav1.ObjectMeta{},
		Spec: v1.PodSpec{
			ProviderId:     pod.Spec.ProviderId,
			RegionId:       pod.Spec.RegionId,
			AccountName:    pod.Spec.AccountName,
			Volumes:        nil,
			InitContainers: nil,
			Containers:     pod.Spec.Containers,
		},
		Status: v1.PodStatus{
			Phase:                      "",
			Conditions:                 nil,
			Message:                    "",
			Reason:                     "",
			NominatedNodeName:          "",
			HostIP:                     "",
			PodIP:                      "",
			PodIPs:                     nil,
			StartTime:                  nil,
			InitContainerStatuses:      nil,
			ContainerStatuses:          nil,
			QOSClass:                   "",
			EphemeralContainerStatuses: nil,
		},
	}

	return
}

// Update takes the representation of a pcm_pod and updates it. Returns the server's representation of the pcm_pod, and an error, if there is any.
func (c *pods) Update(ctx context.Context, pod *v1.Pod, opts metav1.UpdateOptions) (result *v1.Pod, err error) {
	result = &v1.Pod{}

	provider := pbtenant.CloudProvider(pod.Spec.ProviderId)
	var configFile string
	flag.StringVar(&configFile, "confHuawei", "configs/tenanter.yaml", "tenanter.yaml")
	flag.Parse()
	defer glog.Flush()

	if err := tenanter.LoadCloudConfigsFromFile(configFile); err != nil {
		if !errors.Is(err, tenanter.ErrLoadTenanterFileEmpty) {
			glog.Fatalf("tenanter.LoadCloudConfigsFromFile error %+v", err)
		}
		glog.Warningf("tenanter.LoadCloudConfigsFromFile empty file path %s", configFile)
	}

	glog.Infof("load tenant from file finished")
	tenanters, err := tenanter.GetTenanters(provider)
	var regionId int32
	var cpuPod string
	var memoryPod string

	//华为(K8S)与其他厂商的CPU单位和格式有区别
	switch pod.Spec.ProviderId {
	case 0:
		regionId, _ = tenanter.GetAliRegionId(pod.Spec.RegionId)
		cpuPod = strconv.FormatInt(pod.Spec.Containers[0].Resources.Limits.Cpu().MilliValue()/1000, 10)
		memoryPod = strconv.FormatInt(pod.Spec.Containers[0].Resources.Limits.Memory().MilliValue()/1024/1024/1024/1000, 10)
	case 1:
		regionId, _ = tenanter.GetTencentRegionId(pod.Spec.RegionId)
		cpuPod = strconv.FormatInt(pod.Spec.Containers[0].Resources.Limits.Cpu().MilliValue()/1000, 10)
		memoryPod = strconv.FormatInt(pod.Spec.Containers[0].Resources.Limits.Memory().MilliValue()/1024/1024/1024/1000, 10)
	case 2:
		regionId, _ = tenanter.GetHuaweiRegionId(pod.Spec.RegionId)
		cpuPod = pod.Spec.Containers[0].Resources.Limits.Cpu().String()
		memoryPod = pod.Spec.Containers[0].Resources.Limits.Memory().String()
	case 3:
		regionId, _ = tenanter.GetK8SRegionId(pod.Spec.RegionId)
		cpuPod = strconv.FormatInt(pod.Spec.Containers[0].Resources.Limits.Cpu().MilliValue()/1000, 10)
		memoryPod = strconv.FormatInt(pod.Spec.Containers[0].Resources.Limits.Memory().MilliValue()/1024/1024/1024/1000, 10)
	}

	requestPCM := &pbpod.UpdatePodReq{
		RequestSource:  "huawei",
		Provider:       pbpod.CloudProvider(provider),
		PcmId:          string(pod.ObjectMeta.UID),
		PodId:          string(pod.ObjectMeta.UID),
		AccountName:    tenanters[0].AccountName(),
		PodName:        pod.Name,
		RegionId:       regionId,
		ContainerImage: pod.Spec.Containers[0].Image,
		ContainerName:  pod.Spec.Containers[0].Name,
		CpuPod:         cpuPod,
		MemoryPod:      memoryPod,
		Namespace:      pod.ObjectMeta.Namespace,
	}

	server.UpdatePod(nil, requestPCM)

	result = &v1.Pod{
		TypeMeta:   metav1.TypeMeta{},
		ObjectMeta: metav1.ObjectMeta{},
		Spec: v1.PodSpec{
			ProviderId:                    pod.Spec.ProviderId,
			RegionId:                      pod.Spec.RegionId,
			AccountName:                   pod.Spec.AccountName,
			Volumes:                       nil,
			InitContainers:                nil,
			Containers:                    pod.Spec.Containers,
			EphemeralContainers:           nil,
			RestartPolicy:                 "",
			TerminationGracePeriodSeconds: nil,
			ActiveDeadlineSeconds:         nil,
			DNSPolicy:                     "",
			NodeSelector:                  nil,
			ServiceAccountName:            "",
			DeprecatedServiceAccount:      "",
			AutomountServiceAccountToken:  nil,
			NodeName:                      "",
			HostNetwork:                   false,
			HostPID:                       false,
			HostIPC:                       false,
			ShareProcessNamespace:         nil,
			SecurityContext:               nil,
			ImagePullSecrets:              nil,
			Hostname:                      "",
			Subdomain:                     "",
			Affinity:                      nil,
			SchedulerName:                 "",
			Tolerations:                   nil,
			HostAliases:                   nil,
			PriorityClassName:             "",
			Priority:                      nil,
			DNSConfig:                     nil,
			ReadinessGates:                nil,
			RuntimeClassName:              nil,
			EnableServiceLinks:            nil,
			PreemptionPolicy:              nil,
			Overhead:                      nil,
			TopologySpreadConstraints:     nil,
			SetHostnameAsFQDN:             nil,
		},
		Status: v1.PodStatus{
			Phase:                      "",
			Conditions:                 nil,
			Message:                    "",
			Reason:                     "",
			NominatedNodeName:          "",
			HostIP:                     "",
			PodIP:                      "",
			PodIPs:                     nil,
			StartTime:                  nil,
			InitContainerStatuses:      nil,
			ContainerStatuses:          nil,
			QOSClass:                   "",
			EphemeralContainerStatuses: nil,
		},
	}
	return
}

// UpdateStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating UpdateStatus().
func (c *pods) UpdateStatus(ctx context.Context, pod *v1.Pod, opts metav1.UpdateOptions) (result *v1.Pod, err error) {
	result = &v1.Pod{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("pods").
		Name(pod.Name).
		SubResource("status").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(pod).
		Do(ctx).
		Into(result)
	return
}

// Delete takes name of the pcm_pod and deletes it. Returns an error if one occurs.
func (c *pods) Delete(ctx context.Context, namespace string, providerId int32, regionName string, accountName string, pcmId string, opts metav1.DeleteOptions) error {

	provider := pbtenant.CloudProvider(providerId)
	var configFile string
	flag.StringVar(&configFile, "confHuawei", "configs/tenanter.yaml", "tenanter.yaml")
	flag.Parse()
	defer glog.Flush()

	if err := tenanter.LoadCloudConfigsFromFile(configFile); err != nil {
		if !errors.Is(err, tenanter.ErrLoadTenanterFileEmpty) {
			glog.Fatalf("tenanter.LoadCloudConfigsFromFile error %+v", err)
		}
		glog.Warningf("tenanter.LoadCloudConfigsFromFile empty file path %s", configFile)
	}

	glog.Infof("load tenant from file finished")
	var regionId int32
	switch providerId {
	case 0:
		regionId, _ = tenanter.GetAliRegionId(regionName)
	case 1:
		regionId, _ = tenanter.GetTencentRegionId(regionName)
	case 2:
		regionId, _ = tenanter.GetHuaweiRegionId(regionName)
	case 3:
		regionId, _ = tenanter.GetK8SRegionId(regionName)
	}

	requestPCM := &pbpod.DeletePodReq{
		RequestSource: "huawei",
		Provider:      pbpod.CloudProvider(provider),
		AccountName:   accountName,
		PcmId:         pcmId,
		Namespace:     namespace,
		RegionId:      regionId,
	}

	_, err := server.DeletePod(nil, requestPCM)
	if err != nil {
		return err
	}

	return nil

}

// DeleteCollection deletes a collection of objects.
func (c *pods) DeleteCollection(ctx context.Context, opts metav1.DeleteOptions, listOpts metav1.ListOptions) error {
	var timeout time.Duration
	if listOpts.TimeoutSeconds != nil {
		timeout = time.Duration(*listOpts.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Namespace(c.ns).
		Resource("pods").
		VersionedParams(&listOpts, scheme.ParameterCodec).
		Timeout(timeout).
		Body(&opts).
		Do(ctx).
		Error()
}

// Patch applies the patch and returns the patched pcm_pod.
func (c *pods) Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts metav1.PatchOptions, subresources ...string) (result *v1.Pod, err error) {
	result = &v1.Pod{}
	err = c.client.Patch(pt).
		Namespace(c.ns).
		Resource("pods").
		Name(name).
		SubResource(subresources...).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(data).
		Do(ctx).
		Into(result)
	return
}

// Apply takes the given apply declarative configuration, applies it and returns the applied pcm_pod.
func (c *pods) Apply(ctx context.Context, pod *corev1.PodApplyConfiguration, opts metav1.ApplyOptions) (result *v1.Pod, err error) {
	if pod == nil {
		return nil, fmt.Errorf("pcm_pod provided to Apply must not be nil")
	}
	patchOpts := opts.ToPatchOptions()
	data, err := json.Marshal(pod)
	if err != nil {
		return nil, err
	}
	name := pod.Name
	if name == nil {
		return nil, fmt.Errorf("pcm_pod.Name must be provided to Apply")
	}
	result = &v1.Pod{}
	err = c.client.Patch(types.ApplyPatchType).
		Namespace(c.ns).
		Resource("pods").
		Name(*name).
		VersionedParams(&patchOpts, scheme.ParameterCodec).
		Body(data).
		Do(ctx).
		Into(result)
	return
}

// ApplyStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating ApplyStatus().
func (c *pods) ApplyStatus(ctx context.Context, pod *corev1.PodApplyConfiguration, opts metav1.ApplyOptions) (result *v1.Pod, err error) {
	if pod == nil {
		return nil, fmt.Errorf("pcm_pod provided to Apply must not be nil")
	}
	patchOpts := opts.ToPatchOptions()
	data, err := json.Marshal(pod)
	if err != nil {
		return nil, err
	}

	name := pod.Name
	if name == nil {
		return nil, fmt.Errorf("pcm_pod.Name must be provided to Apply")
	}

	result = &v1.Pod{}
	err = c.client.Patch(types.ApplyPatchType).
		Namespace(c.ns).
		Resource("pods").
		Name(*name).
		SubResource("status").
		VersionedParams(&patchOpts, scheme.ParameterCodec).
		Body(data).
		Do(ctx).
		Into(result)
	return
}

// GetEphemeralContainers takes name of the pcm_pod, and returns the corresponding v1.EphemeralContainers object, and an error if there is any.
func (c *pods) GetEphemeralContainers(ctx context.Context, podName string, options metav1.GetOptions) (result *v1.EphemeralContainers, err error) {
	result = &v1.EphemeralContainers{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("pods").
		Name(podName).
		SubResource("ephemeralcontainers").
		VersionedParams(&options, scheme.ParameterCodec).
		Do(ctx).
		Into(result)
	return
}

// UpdateEphemeralContainers takes the top resource name and the representation of a ephemeralContainers and updates it. Returns the server's representation of the ephemeralContainers, and an error, if there is any.
func (c *pods) UpdateEphemeralContainers(ctx context.Context, podName string, ephemeralContainers *v1.EphemeralContainers, opts metav1.UpdateOptions) (result *v1.EphemeralContainers, err error) {
	result = &v1.EphemeralContainers{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("pods").
		Name(podName).
		SubResource("ephemeralcontainers").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(ephemeralContainers).
		Do(ctx).
		Into(result)
	return
}
